@@ -1,5 +1,4 @@
-// // [[Rcpp::plugins(cpp11)]]
-#include <Rcpp.h>
+#include <Rcpp/Lighter>
 using namespace Rcpp;
 
 
@@ -105,114 +104,49 @@ SEXP qFCpp(SEXP x, bool ordered = true, bool na_exclude = true, bool keep_attr =
 }
 
 
-// TODO: could still remove NA, and also for sort_unique
-template<int RTYPE>
-Vector<RTYPE> uniqueord(const Vector<RTYPE>& x) {
-  sugar::IndexHash<RTYPE> hash(x);
-  hash.fill();
-  // int l = x.size(); // almost same speed as member fill.
-  // for(int i = 0; i != l; ++i) {
-  //   unsigned int addr = hash.get_addr(hash.src[i]);
-  //   while(hash.data[addr] && hash.not_equal(hash.src[hash.data[addr] - 1], hash.src[i])) {
-  //     ++addr;
-  //     if(addr == static_cast<unsigned int>(hash.m)) addr = 0;
-  //   }
-  //   if(!hash.data[addr]) {
-  //     hash.data[addr] = i+1;
-  //     ++hash.size_;
-  //   }
-  // }
-
-  int hs = hash.size_;
-  IntegerVector ord = no_init_vector(hs);
-  for(int i = 0, j = 0; j < hs; i++) if(hash.data[i]) ord[j++] = hash.data[i]-1;
-  std::sort(ord.begin(), ord.end());
-  Vector<RTYPE> res = no_init_vector(hs);
-  for(int i = 0; i < hs; ++i) res[i] = hash.src[ord[i]];
-  // inline Vector<RTYPE> keys() const{
-  //   Vector<RTYPE> res = no_init(size_) ;
-  //   for( int i=0, j=0; j<size_; i++){
-  //     if( data[i] ) res[j++] = src[data[i]-1] ;
-  //   }
-  //   return res ;
-  // }
-  return res;
-}
-
 
 template <int RTYPE>
-Vector<RTYPE> funiqueImpl(const Vector<RTYPE>& x, bool sort) {
-  if(sort) {
-    Vector<RTYPE> out = sort_unique(x);
-    Rf_copyMostAttrib(x, out);
-    return out;
-  } else {
-    Vector<RTYPE> out = uniqueord<RTYPE>(x);
-    Rf_copyMostAttrib(x, out);
-    return out;
-  }
+Vector<RTYPE> sortuniqueImpl(const Vector<RTYPE>& x) {
+  Vector<RTYPE> out = sort_unique(x);
+  Rf_copyMostAttrib(x, out);
+  return out;
 }
 
 
-IntegerVector funiqueFACT(const IntegerVector& x, bool sort = true) {
+IntegerVector sortuniqueFACT(const IntegerVector& x) {
   int nlevp = Rf_nlevels(x)+1, l = x.size(), k = 0;
   std::vector<bool> not_seen(nlevp, true);
   bool countNA = true;
-  if(sort) {
-    for(int i = 0; i != l; ++i) {
-      if(x[i] == NA_INTEGER) {
-        if(countNA) {
-          ++k;
-          countNA = false;
-        }
-        continue;
-      }
-      if(not_seen[x[i]]) {
-        not_seen[x[i]] = false;
-        if(++k == nlevp) break;
+  for(int i = 0; i != l; ++i) {
+    if(x[i] == NA_INTEGER) {
+      if(countNA) {
+        ++k;
+        countNA = false;
       }
+      continue;
     }
-    IntegerVector out = no_init_vector(k);
-    if(!countNA) out[k-1] = NA_INTEGER;
-    k = 0;
-    for(int i = 1; i != nlevp; ++i) if(!not_seen[i]) out[k++] = i;
-    Rf_copyMostAttrib(x, out);
-    return out;
-  } else {
-    IntegerVector uxp = no_init_vector(nlevp);
-    for(int i = 0; i != l; ++i) {
-      if(x[i] == NA_INTEGER) {
-        if(countNA) {
-          uxp[k++] = NA_INTEGER;
-          countNA = false;
-        }
-        continue;
-      }
-      if(not_seen[x[i]]) {
-        uxp[k++] = x[i];
-        if(k == nlevp) {
-          Rf_copyMostAttrib(x, uxp);
-          return uxp;
-        }
-        not_seen[x[i]] = false;
-      }
+    if(not_seen[x[i]]) {
+      not_seen[x[i]] = false;
+      if(++k == nlevp) break;
     }
-    IntegerVector out = no_init_vector(k);
-    for(int i = 0; i != k; ++i) out[i] = uxp[i];
-    Rf_copyMostAttrib(x, out);
-    return out;
   }
+  IntegerVector out = no_init_vector(k);
+  if(!countNA) out[k-1] = NA_INTEGER;
+  k = 0;
+  for(int i = 1; i != nlevp; ++i) if(!not_seen[i]) out[k++] = i;
+  Rf_copyMostAttrib(x, out);
+  return out;
 }
 
 
 // [[Rcpp::export]]
-SEXP funiqueCpp(SEXP x, bool sort = true) {
+SEXP sortuniqueCpp(SEXP x) {
   switch(TYPEOF(x)) {
   case INTSXP:
-    if(Rf_isFactor(x)) return funiqueFACT(x, sort);
-    return funiqueImpl<INTSXP>(x, sort);
-  case REALSXP: return funiqueImpl<REALSXP>(x, sort);
-  case STRSXP: return funiqueImpl<STRSXP>(x, sort);
+    if(Rf_isFactor(x)) return sortuniqueFACT(x);
+    return sortuniqueImpl<INTSXP>(x);
+  case REALSXP: return sortuniqueImpl<REALSXP>(x);
+  case STRSXP: return sortuniqueImpl<STRSXP>(x);
   case LGLSXP: {
     LogicalVector xl = x;
     int nc = 0, n0 = 0, n1 = 0, n2 = 0, l = xl.size();
@@ -227,17 +161,10 @@ SEXP funiqueCpp(SEXP x, bool sort = true) {
       if(nc == 3) break;
     }
     LogicalVector out = no_init_vector(nc);
-    if(sort) {
-      nc = 0;
-      if(n0) out[nc++] = false;
-      if(n1) out[nc++] = true;
-      if(n2) out[nc] = NA_LOGICAL;
-    } else {
-      if(n0) out[n0-1] = false;
-      if(n1) out[n1-1] = true;
-      if(n2) out[n2-1] = NA_LOGICAL;
-    }
-    // LogicalVector::create(false, true, NA_LOGICAL)[nd];
+    nc = 0;
+    if(n0) out[nc++] = false;
+    if(n1) out[nc++] = true;
+    if(n2) out[nc] = NA_LOGICAL;
     Rf_copyMostAttrib(x, out);
     return out;
   }
@@ -294,7 +221,7 @@ IntegerVector fdroplevelsCpp(const IntegerVector& x, bool check_NA = true) {
   } else {
     for(int i = 0; i != l; ++i) out[i] = uxp[x[i]];
   }
-  DUPLICATE_ATTRIB(out, x);
+  SHALLOW_DUPLICATE_ATTRIB(out, x);
   Rf_setAttrib(out, R_LevelsSymbol, newlevs);
   return out;
 }
